@@ -81,6 +81,15 @@ public:
 		const char *value;
 	} define;
 
+	class __EXPORT keyclone : public treemap<char *>
+	{
+	public:
+		void enlist(keyclone *trunk);
+
+		inline void reset(const char *tag)
+			{id = (char *)tag;};
+	};
+
 	class __EXPORT usernode
 	{
 	public:
@@ -248,9 +257,7 @@ protected:
 
 	void setHeader(const char *header);
 	void clearId(void);
-
-private:
-	void __LOCAL addAttributes(keynode *node, char *astr);
+	void addAttributes(keynode *node, char *astr);
 };
 
 #define	RUNLEVELS	(sizeof(callback::runlevels) / sizeof(LinkedObject *))
