@@ -17,6 +17,11 @@
 #include <signal.h>
 #include <ctype.h>
 
+#ifdef	HAVE_PWD_H
+#include <pwd.h>
+#include <grp.h>
+#endif
+
 #ifdef	HAVE_NET_IF_H
 #include <net/if.h>
 #include <arpa/inet.h>
@@ -62,6 +67,10 @@ static bool activating(int argc, char **args)
 	return rtn;
 }
 
+unsigned server::uid = 1000;
+const char *server::sipusers = "sipusers";
+const char *server::sipadmin = "wheel";
+
 server::server(const char *id) :
 service(id, PAGING_SIZE)
 {
@@ -265,6 +274,7 @@ void server::confirm(const char *user)
 	string_t digest;
 	const char *dirpath = ".";
 	const char *fn;
+	char *cp;
 
 	snprintf(buf, sizeof(buf), "- welcome prefix=%d range=%d", prefix, range);
 	setHeader(buf);
@@ -459,8 +469,10 @@ void server::confirm(const char *user)
 		}
 		else if(leaf && id) {
 			id = leaf->getPointer();
-			if(create(id, *node))
+			if(create(id, *node)) {
 				process::errlog(WARN, "duplicate identity %s", id);
+				node->setPointer((char *)"duplicate");
+			}
 			else {
 				debug(2, "adding %s %s", node->getId(), id);
 				if(!stricmp(node->getId(), "reject"))
@@ -491,6 +503,119 @@ void server::confirm(const char *user)
 		}
 		node.next();	
 	}
+
+#ifdef	HAVE_PWD_H
+	count = 0;
+	char *member;
+	const char *tempname;
+	keynode *base = getPath("accounts");
+	keyclone *clone, *entry;
+	linked_pointer<keynode> temp;
+
+	struct passwd *pwd;
+	struct group *grp = NULL; 
+
+	if(sipusers)
+		grp = getgrnam(sipusers);
+
+	// if no separated privilege, then use sipadmin
+	if(!grp)
+		grp = getgrnam(sipadmin);
+
+	if(grp && !grp->gr_mem)
+		return;
+
+	// if no groups at all, try to add base user if enabled....
+	// this allows fully automated service for primary desktop user...
+	if(!grp && uid) {
+		pwd = getpwuid(uid);
+		if(!pwd)
+			return;
+		leaf = addNode(base, "user", NULL);
+		addNode(leaf, "id", pwd->pw_name);
+	}
+	else while(NULL != (member = grp->gr_mem[count++])) {
+		leaf = addNode(base, "user", NULL);
+		addNode(leaf, "id", member); 
+	}
+
+	grp = getgrnam(sipadmin);
+
+	node = base->getFirst();
+	while(is(node)) {
+		count = 0;
+		id = NULL;
+		leaf = node->leaf("id");
+		if(leaf && leaf->getPointer())
+			id = leaf->getPointer();
+
+		pwd = NULL;
+		if(id)
+			pwd = getpwnam(id);
+		
+		if(!pwd) {
+			node->setPointer((char *)"invalid");
+			goto skip;
+		}
+
+		if(create(id, *node)) {
+			node->setPointer((char *)"duplicate");
+			goto skip;
+		}
+
+		id = pwd->pw_gecos;
+
+		cp = strchr(id, ',');
+		if(cp)
+			*(cp++) = 0;
+		addNode(*node, "display", id);
+		if(cp)
+			number = atoi(cp);
+		else
+			number = 0;
+		if(!number && pwd->pw_uid >= uid && uid > 0)
+			number = pwd->pw_uid - uid + prefix;
+
+		if(number >= prefix && number < prefix + range && extmap[number - prefix] == NULL)
+			extmap[number - prefix] = *node;
+
+		if(number) {
+			snprintf(buf, 16, "%d", number);
+			addNode(*node, "extension", buf);
+		}
+
+		entry = (keyclone *)(*node);
+		count = 0;
+		tempname = "templates.user";
+		while(grp && grp->gr_mem && NULL != (member = grp->gr_mem[count++])) {
+			if(String::equal(member, pwd->pw_name)) {
+				tempname = "templates.admin";
+				entry->reset("admin");
+				break;
+			}
+		}
+		
+		debug(2, "adding %s %s", node->getId(), pwd->pw_name);
+
+		clone = (keyclone *)getPath(tempname);
+		if(!clone)
+			goto skip;
+
+		temp = clone->getFirst();
+		while(is(temp)) {
+			clone = (keyclone *)alloc(sizeof(keynode));
+			memcpy(clone, *temp, sizeof(keynode));
+			clone->enlist(entry);
+			temp.next();
+		}
+
+skip:
+		node.next();
+	}	
+	
+	endpwent();
+	endgrent();
+#endif
 }
 
 void server::release(keynode *node)
@@ -640,6 +765,26 @@ service::keynode *server::getRouting(const char *id)
 	return NULL;
 }
 
+bool server::checkId(const char *uid)
+{
+	assert(uid != NULL && *uid != 0);
+	assert(cfg != NULL);
+	keynode *node = NULL;
+	server *cfgp;
+
+	locking.access();
+	cfgp = static_cast<server*>(cfg);
+	if(!cfgp) {
+		locking.release();
+		return false;
+	}
+	node = cfgp->find(uid);
+	locking.release();
+	if(node)
+		return true;
+	return false;
+}
+
 void server::getDialing(const char *uid, usernode& user)
 {
 	assert(uid != NULL && *uid != 0);
@@ -891,6 +1036,9 @@ void server::run(const char *user)
 	if(dt->tm_year < 1900)
 		dt->tm_year += 1900;
 
+	// initial load of digest cache
+	digest::load();
+
 	process::printlog("server starting %04d-%02d-%02d %02d:%02d:%02d\n",
 		dt->tm_year, dt->tm_mon + 1, dt->tm_mday,
 		dt->tm_hour, dt->tm_min, dt->tm_sec);
@@ -958,6 +1106,15 @@ invalid:
 			continue;
 		}
 
+		if(!stricmp(argv[0], "digest")) {
+			if(argc != 3)
+				goto invalid;
+
+			if(!digest::set(argv[1], argv[2]))
+				process::reply("invalid digest");			
+			continue;
+		}
+
 		if(!stricmp(argv[0], "realm")) {
 			if(argc != 2)
 				goto invalid;
